note
	description: "Robust client for client-server socket program"
	date: "$Date: 2009-05-08 11:53:02 -0700 (Fri, 08 May 2009) $"
	revision: "$Revision: 78576 $"

class
	APPLICATION

inherit
	INET_ADDRESS_FACTORY

inherit {NONE}
	EXECUTION_ENVIRONMENT
		rename
			command_line as ee_command_line
		end

	EXCEPTIONS

create
	make

feature {NONE} -- Initialization

	host: STRING_8 = "localhost"

	port: INTEGER_32 = 12111

	address: detachable INET_ADDRESS

	socket: NETWORK_STREAM_SOCKET

	make
			-- create socket to server and communicate
		do
			set_ipv4_stack_preferred (True) -- ipv4 still the default
			print_host_and_port
			create_socket
			robust_socket_connect
			Io.put_string ("finish echo_client")
			Io.put_new_line
		end

feature {NONE} --Implementation

	print_host_and_port
			-- print host and port to be used
		do
			Io.put_string ("start echo_client, host = ")
			Io.put_string (Host)
			Io.put_string (", port = ")
			Io.put_integer (Port)
			Io.put_new_line
		end

	create_socket
			-- create socket from inet address
		do
			address := create_from_name (Host)
			check
				attached address as a
			then
				create socket.make_client_by_address_and_port (a, Port)
			end
		end

	robust_socket_connect
			-- connect socket; otherwise retry one second later
		local
			quit: BOOLEAN
			m: STRING_8
			problem : BOOLEAN
		do
			if not problem then

				from
					socket.connect -- does not block
				until
					quit
				loop
					print("Enter a message:%N")
					io.read_line
--					send_message (socket, io.last_string.twin)
					send_message_and_receive_reply (socket,io.last_string.twin) -- block
--					send_message_and_receive_reply (socket, "This")
--					send_message_and_receive_reply (socket, "is")
--					send_message_and_receive_reply (socket, "a")
--					send_message_and_receive_reply (socket, "test")
--					if io.last_string ~ "quit" then
--						send_message (socket, "quit")
--						socket.close
--						quit := true
--					end
				end
			end

		rescue
			if not socket.is_connected then
					Io.put_string ("Unable to connect to host " + Host + ":" + Port.out)
					Io.put_new_line
					Io.put_string ("Do you want to retry? [y/n]%N")
					Io.read_line
					m := io.last_string.twin
					if m [1] = 'y' then
						sleep (1000000000)

					else
						quit := True
						print ("Exiting ...%N")
						Io.read_line
						problem := true
					end

					retry
			end
		end

	send_message_and_receive_reply (a_socket: SOCKET; message: STRING_8)
		require
			valid_socket: a_socket /= Void and then a_socket.is_open_read and then a_socket.is_open_write
			valid_message: message /= Void and then not message.is_empty
		local
			m: MESSAGE
			t: TUPLE [BOOLEAN, BOOLEAN]
		do
			create m.make ([], [])
			if  message ~ "0" then
				create m.make ([true, false], [])
			elseif message ~ "1" then
				create m.make ([false, false], [])
			elseif message ~ "2" then
				create m.make ([false, true], [])
			elseif message ~ "3" then
				create m.make ([false, false], [])
			end
			send_message (a_socket, m)
			receive_reply (a_socket)
		end

	send_message (a_socket: SOCKET; message: MESSAGE)
		require
			valid_socket: a_socket /= Void and then a_socket.is_open_write
--			valid_message: message /= Void and then not message.is_empty
		do
--			a_socket.put_string (message + "%N")
--			a_socket.general_store (message)
			message.general_store (a_socket)
		end

	receive_reply (a_socket: SOCKET)
		require
			valid_socket: a_socket /= Void and then a_socket.is_open_read
		local
			l_last_string: detachable STRING_8
		do
			a_socket.read_line
			l_last_string := a_socket.last_string
			check
				l_last_string_attached: l_last_string /= Void
			end
			Io.put_string ("Server Says: ")
			Io.put_string (l_last_string)
			Io.put_new_line
		end

end -- class APPLICATION

