note
	description: "Robust client for client-server socket program"
	date: "$Date: 2009-05-08 11:53:02 -0700 (Fri, 08 May 2009) $"
	revision: "$Revision: 78576 $"

class
	APPLICATION

inherit
	INET_ADDRESS_FACTORY
	STORABLE
--	THREAD

inherit {NONE}
	EXECUTION_ENVIRONMENT
		rename
			command_line as ee_command_line
		end

	EXCEPTIONS

	STATE_CONSTANTS

create
	make

feature {NONE} -- Initialization

	host: STRING_8 = "localhost"

	port: INTEGER_32 = 12111

	address: detachable INET_ADDRESS

	socket: NETWORK_STREAM_SOCKET

	make
			-- create socket to server and communicate
		do
			set_ipv4_stack_preferred (True) -- ipv4 still the default
			print_host_and_port
			create_socket
			robust_socket_connect
			Io.put_string ("finish echo_client")
			Io.put_new_line
		end

feature {NONE} --Implementation

	print_host_and_port
			-- print host and port to be used
		do
			Io.put_string ("start echo_client, host = ")
			Io.put_string (Host)
			Io.put_string (", port = ")
			Io.put_integer (Port)
			Io.put_new_line
		end

	create_socket
			-- create socket from inet address
		do
			address := create_from_name (Host)
			check
				attached address as a
			then
				create socket.make_client_by_address_and_port (a, Port)
			end
		end

	robust_socket_connect
			-- connect socket; otherwise retry one second later
		local
			quit: BOOLEAN
			m: STRING_8
			problem : BOOLEAN
			i: INTEGER
			s: STATE
		do
			if not problem then
				from
					socket.connect -- does not block
				until
					quit
				loop
					print("Enter 0, 1, 2 or 3: %N")
					io.read_line
					i := io.last_integer
					check socket.is_connected end
					check socket.is_open_read end
					check socket.is_open_write end
					-- send plant input state on socket
					inspect i
					when 0 then plant_input_tf.general_store (socket)
					when 1 then plant_input_ff.general_store (socket)
					when 2 then plant_input_ft.general_store (socket)
					when 3 then plant_input_tt.general_store (socket)
					end
					print("On socket to plant, sent input state for i = " + i.out + "%N")

					-- wait to receive plant output state on socket
					check
						attached {MESSAGE} retrieved (socket) as plant_output_state -- blocking
					then
						print("x1, x2 = ")
						print(plant_output_state.output.x1 + ", " + plant_output_state.output.x2)
						io.new_line
					end
				end
			end

		rescue
			if not socket.is_connected then
					Io.put_string ("Unable to connect to host " + Host + ":" + Port.out)
					Io.put_new_line
					Io.put_string ("Do you want to retry? [y/n]%N")
					Io.read_line
					m := io.last_string.twin
					if m [1] = 'y' then
						sleep (1000000000)

					else
						quit := True
						print ("Exiting ...%N")
						Io.read_line
						problem := true
					end

					retry
			end
		end

	send_message_and_receive_reply (a_socket: SOCKET; message: STRING_8)
		require
			valid_socket: a_socket /= Void and then a_socket.is_open_read and then a_socket.is_open_write
			valid_message: message /= Void and then not message.is_empty
		local
--			m: MESSAGE
			plant_input: STATE
--			t: TUPLE [BOOLEAN, BOOLEAN]
		do
--			create m.make ([], [])
			create plant_input.make
			if  message ~ "0" then
--				create m.make ([true, false], [])
				create plant_input.make
				plant_input.set_input_state ([true, false])
			elseif message ~ "1" then
--				create m.make ([false, false], [])
				create plant_input.make
				plant_input.set_input_state ([false, false])
			elseif message ~ "2" then
--				create m.make ([false, true], [])
				create plant_input.make
				plant_input.set_input_state ([false, true])
			elseif message ~ "3" then
				create plant_input.make
				plant_input.set_input_state ([true, true])
			end
			send_message (a_socket, plant_input)
			receive_reply (a_socket)
		end

	send_message (a_socket: SOCKET; message: STATE)
		require
			valid_socket: a_socket /= Void and then a_socket.is_open_write
--			valid_message: message /= Void and then not message.is_empty
		do
--			a_socket.put_string (message + "%N")
--			a_socket.general_store (message)
			message.general_store (a_socket)
		end

	receive_reply (a_socket: SOCKET)
		require
			valid_socket: a_socket /= Void and then a_socket.is_open_read
		local
			l_last_string: detachable STRING_8
		do
--			a_socket.read_line
--			l_last_string := a_socket.last_string
			check
				attached {STATE} retrieved (a_socket) as plant_output_state
			then
--			check
--				l_last_string_attached: l_last_string /= Void
--			end
			Io.put_string ("Server Says: ")
			print(plant_output_state.out)
			io.new_line
			end
--			Io.put_string (l_last_string)
--			Io.put_new_line
		end

	plant_input_tf: MESSAGE
		do
			create Result.make([true, false], [off, closed])
--			Result.set_input_state([true, false])
		end

	plant_input_ff: MESSAGE
		do
				create Result.make([false, false], [off, closed])
--			Result.set_input_state([false, false])
		end

	plant_input_ft: MESSAGE
		do
				create Result.make([false, true], [off, closed])
--			Result.set_input_state([false, true])
		end

	plant_input_tt: MESSAGE
		do
			create Result.make([true, true], [off, closed])
--			Result.set_input_state([true, true])
		end


end -- class APPLICATION

