note
	description: "Summary description for {COMMUNICATION}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	COMM_MANAGER
inherit
	THREAD

	INET_PROPERTIES

	THREAD

	STORABLE


create
	make

feature {NONE} -- Initialization

	state: STATE

	port: INTEGER_32 = 12111

	plant_socket: NETWORK_STREAM_SOCKET

	controller_socket: detachable NETWORK_STREAM_SOCKET

	make(s: STATE)
		do
			Io.put_string ("starting plant server, port = " + Port.out + "%N")
			set_ipv4_stack_preferred (True)
			create plant_socket.make_server_by_port (Port)
			state := s
			launch
		end

	print_addresses
		do
			check
				attached controller_socket as cs
			then
				check
					attached cs.address as csa and attached cs.peer_address as cspa
				then
					Io.put_string ("Accepted client on the listen socket address = " + csa.host_address.host_address + " port = " + csa.port.out + ".")
					Io.put_new_line
					Io.put_string ("%T Accepted client address = " + cspa.host_address.host_address + " , port = " + cspa.port.out)
					Io.put_new_line
				end
			end
		end

feature

	run_simulation : BOOLEAN = false

	execute
		local
			done: BOOLEAN
			problem: BOOLEAN
			plant_input_tuple: TUPLE[b1:BOOLEAN; b2:BOOLEAN]
			plant_output_tuple: TUPLE[x1:STRING; x2: STRING]
			plant_output_message: MESSAGE
		do

			if not problem then
				if run_simulation then open_loop_simulation

				else -- get inputs from controller


					--start socket
					plant_socket.listen (1)
					plant_socket.accept -- block

					controller_socket := plant_socket.accepted

					-- if connected, get plant inputs, and transmit plant outputs
					check
						attached controller_socket as cs
					then
						print_addresses
						from
							done := False
						until
							done
						loop
							check
								-- block until able to retrieve plant input state
								attached {MESSAGE} retrieved (cs) as plant_input_state
							then
								plant_input_tuple := plant_input_state.input.deep_twin
								print(plant_input_tuple)

								-- update state
								print("%NCOM_MANAGER state try lock%N")

								-- lock the global state, update it, record it and unlock
								-- do this quickly
								state.mutex.lock
								state.set_input_state(plant_input_tuple)
								plant_output_tuple := state.output.deep_twin
								state.mutex.unlock

								print("%NCOM_MANAGER state unlocked%N")

								-- send plant state to controller socket
								create plant_output_message.make ([false, false], plant_output_tuple)
								plant_output_message.general_store (cs)

								sleep(10000000000)
							end
						end
					end -- simulation
				end
			end
			plant_socket.close
			Io.put_string ("finish echo_server%N")
		rescue
			print("Exception is: " + exception.out)
			if attached controller_socket as cs then cs.close end
			if attached plant_socket as ls then ls.close end
			problem := true
			retry
		end

feature -- open-loop

	open_loop_simulation
		local
			time: INTEGER -- in seconds
		do
				time := 10
--				sleep(time*1000000000)
				print("%NStart simulation%N")

				-- start motor
				state.mutex.lock
				state.set_input_state ([true, false])
				state.mutex.unlock

				sleep(time*1000000000)

				-- stop motor
				state.mutex.lock
				state.set_input_state ([false, false])
				state.mutex.unlock

				sleep(time*1000000000)

				-- open gate
				state.mutex.lock
				state.set_input_state ([false, true])
				state.mutex.unlock

				sleep(time*1000000000)

				-- close gate
				state.mutex.lock
				state.set_input_state ([false, false])
				state.mutex.unlock

				print("%NSimulation finished%N")
		end


end
