note
	description: "Summary description for {STATE_MACHINE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	STATE_MACHINE

inherit {NONE}
	STATE_CONSTANTS

create
	make

feature -- state and inputs

	state: STATE
			-- global state as once
		once
			create Result.make
		end

	set_motor_guard (b: BOOLEAN)
		do
			State.input.b1 := b
		end

	set_door_guard (b: BOOLEAN)
		do
			State.input.b2 := b
		end

feature {NONE} -- init

	make
			-- state is  motor_guard, motor_state door_guard, door_state,
		local
			event: EVENT
		do
			-- intialize the state
			State.set_input_state ([False, False])
			State.set_output_state ([Off, Closed])

			-- create events and launch them
			create event.make (State, "motor_start", agent motor_start_guard (State), agent motor_start_action (State))
			event.launch
			create event.make (State, "motor_on",  agent motor_on_guard (State), agent motor_on_action (State))
			event.launch
			create event.make (State, "motor_stop", agent motor_stop_guard (State), agent motor_stop_action (State))
			event.launch
			create event.make (State, "motor_off", agent motor_off_guard (State), agent motor_off_action (State))
			event.launch
			create event.make (State, "door_opening", agent door_opening_guard (State), agent door_opening_action (State))
			event.launch
			create event.make (State, "door_open", agent door_open_guard (State), agent door_open_action (State))
			event.launch
			create event.make (State, "door_closing", agent door_closing_guard (State), agent door_closing_action (State))
			event.launch
			create event.make (State, "door_close", agent door_close_guard (State), agent door_close_action (State))
			event.launch
		end



feature -- motor guards and events

	motor_start_guard (s: like State): BOOLEAN
		do
			Result := s.input.b1 = True and s.output.x1 ~ Off
		end

	motor_start_action (s: like State)
		do
			s.output.x1 := Starting
		end

	motor_on_guard (s: like State): BOOLEAN
		do
			Result := s.output.x1 ~ Starting
		end

	motor_on_action (s: like State)
		do
			s.output.x1 := On
		end

	motor_stop_guard (s: like State): BOOLEAN
		do
			Result := s.input.b1 = False and s.output.x1 ~ On
		end

	motor_stop_action (s: like State)
		do
			s.output.x1 := Stopping
		end

	motor_off_guard (s: like State): BOOLEAN
		do
			Result := s.output.x1 ~ Stopping
		end

	motor_off_action (s: like State)
		do
			s.output.x1 := Off
		end

feature -- door guards and events

	door_opening_guard (s: like State): BOOLEAN
		do
			Result := s.input.b2 = True and s.output.x2 ~ Closed
		end

	door_opening_action (s: like State)
		do
			s.output.x2 := Opening
		end

	door_open_guard (s: like State): BOOLEAN
		do
			Result := s.output.x2 ~ Opening
		end

	door_open_action (s: like State)
		do
			s.output.x2 := Open
		end

	door_closing_guard (s: like State): BOOLEAN
		do
			Result := s.input.b2 = False and s.output.x2 ~ Open
		end

	door_closing_action (s: like State)
		do
			s.output.x2 := Closing
		end

	door_close_guard (s: like State): BOOLEAN
		do
			Result := s.output.x2 ~ Closing
		end

	door_close_action (s: like State)
		do
			s.output.x2 := Closed
		end

invariant
	State.output.x2 ~ Closed or State.output.x1 ~ Off
	is_motor_state (State.output.x1) and is_door_state (State.output.x2)

end -- class STATE_MACHINE

